<?php

namespace Insign\PostCore\Model;

use Insign\PostCore\Api\AddressValidationInterface;
use Insign\PostCore\Helper\TokenHelper;
use Insign\PostCore\Util\AddressMapper;
use Insign\PostCore\Util\OAuth2\AbstractProvider;
use Insign\PostCore\Util\OAuth2\AccessTokenManager;
use Insign\PostCore\Util\OAuth2\AddressValidationProvider;
use Magento\Checkout\Model\Session;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Quote\Api\Data\AddressInterface;
use Magento\Store\Model\ScopeInterface;

class AddressValidationManagement implements AddressValidationInterface
{
    /**
     * @var int
     */
    private $requestCounter = 0;

    /**
     * @var AccessTokenManager
     */
    protected $accessTokenManager;
    /**
     * @var AddressValidationProvider
     */
    protected $oauth2Provider;
    /**
     * @var AddressMapper
     */
    protected $addressMapper;
    /**
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;
    /**
     * @var Session
     */
    private $checkoutSession;
    /**
     * @var TokenHelper
     */
    private $tokenHelper;

    /**
     * GuestAddressValidationManagement constructor.
     * @param AccessTokenManager $accessTokenManager
     * @param AddressValidationProvider $oauth2Provider
     * @param AddressMapper $addressMapper
     * @param ScopeConfigInterface $scopeConfig
     * @param Session $checkoutSession
     * @param TokenHelper $tokenHelper
     */
    public function __construct(
        AccessTokenManager $accessTokenManager,
        AddressValidationProvider $oauth2Provider,
        AddressMapper $addressMapper,
        ScopeConfigInterface $scopeConfig,
        Session $checkoutSession,
        TokenHelper $tokenHelper
    ) {
        $this->accessTokenManager = $accessTokenManager;
        $this->oauth2Provider = $oauth2Provider;
        $this->addressMapper = $addressMapper;
        $this->scopeConfig = $scopeConfig;
        $this->checkoutSession = $checkoutSession;
        $this->tokenHelper = $tokenHelper;
    }

    /**
     * @inheritdoc
     * @throws \Exception
     */
    public function validateByAddress($cartId, AddressInterface $address): string
    {
        if (!$this->tokenHelper->isValidationEnable()) {
            return false;
        }

        $options = [];
        $streetNumber = '';
        $street = '';
        if (preg_match('/^([^\d]*[^\d\s]) *(\d.*)$/', $address->getStreet()[0], $match)) {
            if (is_array($match)) {
                $streetNumber = end($match);
                $arrayStreet = array_slice($match, 1, 1, true);
                $street = reset($arrayStreet);
            }
        }
//        else {
//            if (isset($address->getStreet()[1])) {
//                $arrayStreetHouse = $address->getStreet();
//                $streetNumber = end($arrayStreetHouse);
//                $street = reset($arrayStreetHouse);
//            }
//        }

        $customerData = [
            'firstname' => $address->getFirstname(),
            'lastname' => $address->getLastname(),
            'street' => $street,
            'nr' => $streetNumber,
            'postcode' => $address->getPostcode(),
            'city' => $address->getCity()
        ];
        $addressMapper = $this->addressMapper->getAddressData($customerData);
        $url = $this->scopeConfig->getValue(TokenHelper::XML_SWISS_POST_ADDRESS_VALIDATION_ENDPOINT, ScopeInterface::SCOPE_WEBSITE);
        $options['json'] = $addressMapper;
        $accessToken = $this->accessTokenManager->getAccessToken(TokenHelper::XML_SWISS_POST_ADDRESS_VALIDATION_SCOPE);
        if (!isset($accessToken)) {
            $accessToken = $this->oauth2Provider->createAccessToken();
        }

        try {
            $response = $this->oauth2Provider->createAuthenticatedRequest(AbstractProvider::METHOD_HTTP_POST, $url, $accessToken, $options);

            if($response->getStatusCode() == 401 && $this->requestCounter < 1){
                $this->requestCounter++;
                $this->oauth2Provider->createAccessToken();
                return $this->validateByAddress($cartId, $address);
            }
            elseif ($response->getStatusCode() == 401 && $this->requestCounter == 1){
                $this->logger->error("Post API Response: " . $response->getStatusCode());
                return "ERROR";
            }

            $data = json_decode((string)$response->getBody(), true);
            if (isset($data['quality'])) {
                $this->checkoutSession->setVarValue($data['quality']);
                return $data['quality'];

                //  Keep mapping if disabled on frontend (/PostCore/view/frontend/web/js/model/validation-processor.js)
//                switch ($data['quality']) {
//                    case "CERTIFIED":
//                        return "CERTIFIED";
//                    case "USABLE":
//                    case "FIXED":
//                        return "VALID";
//                    case "UNUSABLE":
//                    case "COMPROMISED":
//                        return "NOT VALID";
//                    default:
//                        return $data['quality'];
//                }
            }
        } catch (\Exception $e) {
            if ($e->getCode() == 401 && $this->requestCounter < 1) {
                $this->requestCounter++;
                $this->oauth2Provider->createAccessToken();
                return $this->validateByAddress($cartId, $address);
            } else {
                return "ERROR";
            }
        }
    }
}
