<?php

namespace Insign\PostCore\Model\Config\Source\Barcode;

use Magento\Framework\View\Element\Context;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Shipping\Model\Config;

/**
 * @api
 * @since 100.0.2
 */
class ShippingMethodColumn  extends \Magento\Framework\View\Element\Html\Select
{
    /**
     * @var Config
     */
    private $shippingModelConfig;
    /**
     * @var ScopeConfigInterface
     */
    private $scopeConfig;

    public function __construct(
        Config $shippingModelConfig,
        ScopeConfigInterface $scopeConfig,
        Context $context,
        array $data = []
    ) {
        parent::__construct($context, $data);
        $this->shippingModelConfig = $shippingModelConfig;
        $this->scopeConfig = $scopeConfig;
    }

    /**
     * Set "name" for <select> element
     *
     * @param string $value
     * @return $this
     */
    public function setInputName(string $value): ShippingMethodColumn
    {
        return $this->setName($value);
    }

    /**
     * Set "id" for <select> element
     *
     * @param string $value
     * @return $this
     */
    public function setInputId(string $value): ShippingMethodColumn
    {
        return $this->setId($value);
    }

    /**
     * Render block HTML
     *
     * @return string
     */
    public function _toHtml(): string
    {
        if (!$this->getOptions()) {
            $this->setOptions($this->getSourceOptions());
        }
        return parent::_toHtml();
    }

    /**
     * @return array
     */
    private function getSourceOptions(): array
    {
        $shipping = $this->shippingModelConfig->getActiveCarriers();
        $methods = [];
        foreach ($shipping as $shippingCode => $shippingModel) {
            if ($carrierMethods = $shippingModel->getAllowedMethods()) {
                foreach ($carrierMethods as $methodCode => $method) {
                    $code = $shippingCode . '_' . $methodCode;
                    $carrierTitle = $this->scopeConfig->getValue('carriers/' . $shippingCode . '/title');
                    $methods[] = ['value' => $code, 'label' => $carrierTitle];
                }
            }
        }
        return $methods;
    }

}
