define([
    'jquery',
    'ko',
    'uiRegistry',
    'mage/storage',
    'Insign_PostCore/js/model/swisspost-url-manager',
    'Magento_Checkout/js/model/quote',
    'Insign_PostCore/js/model/validation-processor',
    'Insign_PostCore/js/model/error-processor',
    'Insign_PostCore/js/model/swisspost-messages',
    'uiLayout'
], function ($, ko, registry, storage, apiUrls, quote, validationProcessor, errorProcessor, postcoreMessages) {
    'use strict';

    var mixin = {
        defaults: {
            shippingFormTemplate: 'Insign_PostCore/shipping-address/form',
            imports: {
                selectedCountry: "${ $.name }.shipping-address-fieldset.country_id:value"
            }
        },
        validationFields: ['city', 'postcode', 'street.0', 'firstname', 'lastname'],
        validationCountry: "CH",
        postValidateTimeout: null,
        postValidateCATimeout: null,
        validateDelay: 300,
        addressFormPostValidation: ko.observable(null),
        selectedCountry: ko.observable(null),
        errorMessage: postcoreMessages.error,
        showErrorMessage: postcoreMessages.showError,

        initialize: function () {
            var self = this,
                formPath = 'checkout.steps.shipping-step.shippingAddress.shipping-address-fieldset';
            
            $.each(self.validationFields, function (index, field) {
                registry.async(formPath + '.' + field)(self.doElementBinding.bind(self));
            });

            this.selectedCountry.subscribe(function(value) {
                this.addressFormPostValidation(null);
                errorProcessor.hideError();
                clearTimeout(this.postValidateTimeout);
                if (value === this.validationCountry) {
                    this.postValidateTimeout = setTimeout(this.postAddressValidationHandler.bind(this), this.validateDelay);
                }
            }, this);

            this.showAddressFormPostValidationMessage = ko.computed(function() {
                return (this.selectedCountry() === this.validationCountry);
            }, this);

            quote.shippingAddress.subscribe(function(val){
                if (this.isCustomerLoggedIn()) {
                    postcoreMessages.customerAddressMessage(null);
                    errorProcessor.hideError();
                    if (val.countryId === this.validationCountry) {
                        validationProcessor.isValidating(true);
                        this.postAddessValidationProcessor(val, postcoreMessages.customerAddressMessage);
                    }
                }
            }, this)

            return this._super();
        },

        doElementBinding: function(element, force, delay) {
            var self = this;
            element.on('value', function () {
                self.addressFormPostValidation(null);
                errorProcessor.hideError();
                clearTimeout(self.postValidateTimeout);
                self.postValidateTimeout = setTimeout(self.postAddressValidationHandler.bind(self), self.validateDelay);
            })
        },

        postAddressValidationHandler: function() {
            var self = this,
                hasEmptyFields = false,
                shippingAddressData = registry.get('checkoutProvider').shippingAddress;

            if (shippingAddressData.country_id !== this.validationCountry) return;
            
            $.each(self.validationFields, function (index, field) {
                if (field.indexOf('street') > -1) {
                    if (!shippingAddressData['street'][field.split(".")[1]]) {
                        hasEmptyFields = true;
                    }
                } else {
                    if (!shippingAddressData[field]) {
                        hasEmptyFields = true;
                    }
                }
            });

            if (!hasEmptyFields) {
                this.postAddessValidationProcessor(shippingAddressData);
            }
        },

        postAddessValidationProcessor: function(address, messageContainer) {
            var serviceUrl = apiUrls.getUrlForSwissPostAddressValidation(quote),
                self = this;

            var payload = JSON.stringify({
                address: {
                    'street': address.street,
                    'city': address.city,
                    'region_id': address.regionId,
                    'region': address.region,
                    'country_id': address.countryId,
                    'postcode': address.postcode,
                    'firstname': address.firstname,
                    'lastname': address.lastname,
                    'company': address.company,
                    'telephone': address.telephone,
                }
            });
                
            storage.post(
                serviceUrl,
                payload,
                false
            ).done(function (result) {
                var code = validationProcessor.process(result);
                validationProcessor.isValidating(false);
                if (messageContainer) {
                    messageContainer(code);
                } else {
                    self.addressFormPostValidation(code);
                }
            }).fail(function (response) {
                validationProcessor.isValidating(false);
                if (messageContainer) {
                    messageContainer(null);
                } else {
                    self.addressFormPostValidation(null);
                }
                errorProcessor.process(response);
            });
        }

    };

    return function (target) {
        return target.extend(mixin);
    };

});
